package nckd.jimin.jyyy.fi.plugin.form;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.kingdee.util.StringUtils;
import kd.bos.context.RequestContext;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.form.CloseCallBack;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.schedule.api.JobInfo;
import kd.bos.schedule.api.JobType;
import kd.bos.schedule.api.TaskInfo;
import kd.bos.schedule.form.JobForm;
import nckd.jimin.jyyy.fi.plugin.operate.Helper.PersonReimQuotaHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;

public class LimitRelasetList extends AbstractListPlugin {
    private static final String OP_REFRESH_ALLQUOTA = "refresh_allquota";

    private static final String OP_REFRESH_ALLLEVER = "refresh_alllever";


    private PersonReimQuotaHelper personReimQuotaHelper;

    @Override
    public void initialize() {
        super.initialize();
        personReimQuotaHelper = new PersonReimQuotaHelper();
    }

    protected PersonReimQuotaHelper getPersonReimQuotaHelper() {
        return Optional.ofNullable(personReimQuotaHelper).orElse(new PersonReimQuotaHelper());
    }

    @Override
    public void afterDoOperation(AfterDoOperationEventArgs args) {
        super.afterDoOperation(args);
        String operateKey = args.getOperateKey();
        OperationResult operationResult = args.getOperationResult();
        if(operationResult != null && !operationResult.isSuccess()){
            return;
        }
        if ( OP_REFRESH_ALLQUOTA.equals(operateKey)) {
            dispatch("更新个人额度","updatequota");
        }

        if ( OP_REFRESH_ALLLEVER.equals(operateKey)) {
            dispatch("更新差旅级别","updatelever");
        }
    }
    /**
     * 回调事件，在任务处理完毕后继续后续处理
     */
    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
        if (StringUtils.equals(closedCallBackEvent.getActionId(), "taskcloseback")) {
            Object returnData = closedCallBackEvent.getReturnData();
            if (returnData == null) {
                return;
            }
            if (returnData instanceof Map) {
                Map<String, Object> result = (Map<String, Object>)returnData;
                if (result.containsKey("taskinfo")) {
                    String taskInfoStr = (String)result.get("taskinfo");
                    if (!StringUtils.isEmpty(taskInfoStr)) {
                        TaskInfo taskInfo = JSON.parseObject(taskInfoStr,TaskInfo.class);
                        if (taskInfo.isTaskEnd()) {
                            // 获取任务执行完毕，生成的内容
                            String data = taskInfo.getData();
                            if(!StringUtils.isEmpty(data)){
                                JSONObject dataJson = JSON.parseObject(data);
                                this.getView().showSuccessNotification(dataJson.getString("message"));
                            }else{
                                this.getView().showSuccessNotification("执行成功。");
                            }
                        }
                    }
                }
            }
        }
    }
    /**
     * 创建任务目标，发布新任务
     */
    private void dispatch(String jobName,String updateType) {
        // 创建任务目标
        JobInfo jobInfo = new JobInfo();
        jobInfo.setAppId("em");                // 执行类所在的应用名
        jobInfo.setJobType(JobType.REALTIME);   // 即时执行
        jobInfo.setName(jobName);
        jobInfo.setId(UUID.randomUUID().toString());        // 随机产生一个JobId (任务目标的标识)
        jobInfo.setTaskClassname("nckd.jimin.jyyy.fi.task.PersonReimBigJobTask");
        // 自定义参数
        Map<String,Object> params = new HashMap<>();
        params.put("updatetype", updateType);
        jobInfo.setParams(params);
        jobInfo.setRunByUserId(RequestContext.get().getCurrUserId());
        // 回调参数，设置一个回调处理标识(actionId)
        CloseCallBack closeCallBack = new CloseCallBack(this, "taskcloseback");
        // 发布任务，并显示进度
        JobForm.dispatch(jobInfo, this.getView(), closeCallBack);
    }
}
