package nckd.jimin.jyyy.fi.plugin.operate.Helper;

import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.exception.KDBizException;
import kd.bos.org.utils.DynamicObjectUtils;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.basedata.BaseDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.fi.gl.util.BigDecimalUtil;
import nckd.jimin.jyyy.fi.common.constant.BillTypeConstants;
import nckd.jimin.jyyy.fi.common.constant.LimitRelasetConstant;
import nckd.jimin.jyyy.fi.common.constant.PersonReimSyncTaskConstant;
import nckd.jimin.jyyy.fi.common.util.CommonUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

public class PersonReimQuotaHelper {
    /**
     * 通过 费用项目与职级岗位关系配置 获取额度信息
     * 按照币别分组，允许设置不同币别的方案
     * @return
     */
    public Map<Long, Map<Long, BigDecimal>> getExpenseJobAmount(Object positionId , Object jobLeverId){

        StringJoiner joiner = new StringJoiner(",");
        joiner.add(LimitRelasetConstant.KEY_NCKD_CURRENCY);
        joiner.add(LimitRelasetConstant.KEY_NCKD_EXPENSEITEM);
        joiner.add(LimitRelasetConstant.KEY_NCKD_AMOUNT);

        DynamicObject[] limitRelasetArray = BusinessDataServiceHelper.load(LimitRelasetConstant.ENTITYID, joiner.toString(), new QFilter[]{
                new QFilter(LimitRelasetConstant.KEY_NCKD_POSITIONHR, QCP.equals, positionId),
                new QFilter(LimitRelasetConstant.KEY_NCKD_JOBLEVELHR, QCP.equals, jobLeverId),
                new QFilter(LimitRelasetConstant.KEY_ENABLE, QCP.equals, "1"),
                new QFilter(LimitRelasetConstant.KEY_STATUS, QCP.equals, "C")
        });
        if(limitRelasetArray == null || limitRelasetArray.length == 0){
            return new HashMap<>();
        }
        return Arrays.stream(limitRelasetArray)
                .filter(r -> r.getDynamicObject(LimitRelasetConstant.KEY_NCKD_EXPENSEITEM) != null)
                .collect(Collectors.groupingBy(r -> r.getDynamicObject(LimitRelasetConstant.KEY_NCKD_CURRENCY).getLong(PersonReimSyncTaskConstant.ID),
                        Collectors.toMap(r -> r.getDynamicObject(LimitRelasetConstant.KEY_NCKD_EXPENSEITEM).getLong(PersonReimSyncTaskConstant.ID),
                                r -> r.getBigDecimal(LimitRelasetConstant.KEY_NCKD_AMOUNT), (a, b) -> a)));
    }

    /**
     * 作废条件范围内的个人额度信息
     * @param userId
     * @param companyId
     * @param expenseItemId
     * @param currencyId
     */
    public void doInvalidPersonReim(Long userId , Long companyId , Long expenseItemId ,Long currencyId , int year){
        // 作废币别下
        List<QFilter> filterList = new ArrayList<>();
        if(userId != null && userId == 0L){
            filterList.add(new QFilter("employee.id", QCP.equals, userId));
            if(companyId != null && companyId != 0L){
                filterList.add(new QFilter("company.id", QCP.equals, companyId));
            }
            if(expenseItemId != null && expenseItemId != 0L){
                filterList.add(new QFilter("expenseitem.id", QCP.equals, expenseItemId));
            }
            filterList.add(new QFilter("currency.id", QCP.equals, currencyId));
            filterList.add(new QFilter("auditstatus", QCP.equals, "1"));
            filterList.add(new QFilter("dateyear", QCP.equals, String.valueOf(year)));
            DynamicObject[] invalidPersonReimArray = BusinessDataServiceHelper.load(BillTypeConstants.ER_REIMBURSEAMOUNT, "auditstatus", filterList.toArray(new QFilter[0]));
            for(DynamicObject invalidPersonReim : invalidPersonReimArray){
                invalidPersonReim.set("auditstatus","2");
            }
            SaveServiceHelper.save(invalidPersonReimArray);
        }
    }

    /**
     * 直接创建个人额度信息
     * @param billInfo
     * @param currencyReimJobAmount
     */
    public void createPersonReimDirect(DynamicObject billInfo,Map<Long, Map<Long, BigDecimal>> currencyReimJobAmount){
        for(Map.Entry<Long, Map<Long, BigDecimal>> currencyRow : currencyReimJobAmount.entrySet()){
            Long currencyId = currencyRow.getKey();
            Map<Long, BigDecimal> expenseJobAmount = currencyRow.getValue();
            for(Map.Entry<Long, BigDecimal> dataRow : expenseJobAmount.entrySet()){
                Long expenseItemId = dataRow.getKey();
                BigDecimal amount = dataRow.getValue();
                createPersonReimByExpense(billInfo,currencyId,expenseItemId,amount);
            }
        }
    }

    public void createPersonReimByExpense(DynamicObject billInfo,Long currencyId,Long expenseItemId,BigDecimal amount){

        PersonReimQuotaVO personReimQuotaVO = new PersonReimQuotaVO(
                billInfo.getDynamicObject(PersonReimSyncTaskConstant.KEY_NCKD_USER).getLong(PersonReimSyncTaskConstant.ID),
                billInfo.getDynamicObject(PersonReimSyncTaskConstant.KEY_NCKD_ORG).getLong(PersonReimSyncTaskConstant.ID),
                expenseItemId,
                currencyId,amount,
                billInfo.getInt(PersonReimSyncTaskConstant.KEY_NCKD_YEAR),
                billInfo.getInt(PersonReimSyncTaskConstant.KEY_NCKD_MONTH),
                billInfo.getString(PersonReimSyncTaskConstant.ID));
        // 创建个人额度表
        DynamicObject personReim = createPersonReim(personReimQuotaVO,true);
        // 调用审核
        OperationResult saveOp = OperationServiceHelper.executeOperate("save", BillTypeConstants.ER_REIMBURSEAMOUNT, new DynamicObject[]{personReim}, OperateOption.create());
        if(!saveOp.isSuccess() || saveOp.getSuccessPkIds().size() == 0){
            throw new KDBizException(CommonUtils.getOperationResultErrorInfos(saveOp));
        }
        OperationResult approveOp = OperationServiceHelper.executeOperate("approve", BillTypeConstants.ER_REIMBURSEAMOUNT, new Object[]{saveOp.getSuccessPkIds().get(0)}, OperateOption.create());
        if(!approveOp.isSuccess() || approveOp.getSuccessPkIds().size() == 0){
            throw new KDBizException(CommonUtils.getOperationResultErrorInfos(approveOp));
        }
    }

    /**
     * 获取个人额度同步表 创建某个费用类型的个人额度
     * @param quotaVO
     * @return
     */
    public DynamicObject createPersonReim(PersonReimQuotaVO  quotaVO , boolean isAddnew){

        int year = quotaVO.getYear();
        int month = quotaVO.getMonth();
        BigDecimal amount = quotaVO.getAmount();
        Long userId = quotaVO.getUserId();
        Long companyId = quotaVO.getCompanyId();
        Long expenseItemId = quotaVO.getExpenseItemId();
        Long currencyId = quotaVO.getCurrencyId();

        Date now = new Date();
        long currUserId = RequestContext.get().getCurrUserId();

        // 已与实施确认，通过公司、费用类型、人员、币别、年度确认是否已经创建个人额度表；如果没有则新增，存在则修改
        DynamicObject queryPersonAmount = QueryServiceHelper.queryOne(BillTypeConstants.ER_REIMBURSEAMOUNT, "id", new QFilter[]{
                new QFilter("company.id", QCP.equals, companyId),
                new QFilter("employee.id", QCP.equals, userId),
                new QFilter("expenseitem.id", QCP.equals, expenseItemId),
                new QFilter("currency.id", QCP.equals, currencyId),
                new QFilter("auditstatus", QCP.equals, "1"),
                new QFilter("dateyear", QCP.equals, String.valueOf(year))
        });
        DynamicObject personAmountBill = BusinessDataServiceHelper.newDynamicObject(BillTypeConstants.ER_REIMBURSEAMOUNT);
        // 如果存在则从个人额度中复制
        if(queryPersonAmount != null){
            DynamicObject oriPersonAmount = BusinessDataServiceHelper.loadSingle(queryPersonAmount.get("id"), BillTypeConstants.ER_REIMBURSEAMOUNT);
            Set<String> ignoreSet = new HashSet<>(Arrays.asList("id"));
            DynamicObjectUtils.copy(oriPersonAmount,personAmountBill);
            // 标准字段
            personAmountBill.set("wbsrcbilltype","er_reimctl_modify");
            personAmountBill.set("wbsrcbillid",oriPersonAmount.getPkValue());
            setPersonAmountData(personAmountBill,amount,month);
        }else{
            // 无需新增时直接返回
            if(!isAddnew){
                return null;
            }

            DynamicObject userInfo = BusinessDataServiceHelper.loadSingleFromCache(userId, "bos_user");
            DynamicObject companyInfo = BusinessDataServiceHelper.loadSingleFromCache(companyId, "bos_org");
            DynamicObject expenseItemInfo = BusinessDataServiceHelper.loadSingleFromCache(expenseItemId, "er_expenseitemedit");
            DynamicObject currencyInfo = BusinessDataServiceHelper.loadSingleFromCache(currencyId, "bd_currency");

            personAmountBill.set("company", companyInfo);
            personAmountBill.set("expenseitem", expenseItemInfo);
            personAmountBill.set("currency", currencyInfo);
            personAmountBill.set("dateyear", String.valueOf(year));
            personAmountBill.set("costcompany", companyId);

            personAmountBill.set("amounttype", "1");
            personAmountBill.set("employee", userInfo);
            //personAmountBill.set("dept", reimburseQuota.get("dept.id"));
            personAmountBill.set("auditstatus", "0");
            personAmountBill.set("createtime", now);
            personAmountBill.set("modifytime", now);
            personAmountBill.set("creator", currUserId);
            personAmountBill.set("modifier", currUserId);
            personAmountBill.set("wbsrcbilltype", "er_reimctl_new");
            setPersonAmountData(personAmountBill,amount,1);
        }

        BigDecimal totalAmount = BigDecimal.ZERO;
        // 计算年总额度
        for(int i = 1; i <= 12; i++) {
            totalAmount = BigDecimalUtil.add(totalAmount,personAmountBill.getBigDecimal("month" + i));
        }
        personAmountBill.set("totalamount", totalAmount);
        personAmountBill.set("auditstatus", "0");

        // 岗位新增变动：positionchange ；年度复制：yearcopy  ； 新增字段，与标准字段区分，防止与标品逻辑冲突
        personAmountBill.set("nckd_sourcetype","positionchange");
        personAmountBill.set("nckd_sourcebillid",quotaVO.getSourcebillid());

        return personAmountBill;
    }

    public void syncReimLever(Long userId ,Long companyId,Long jobId){

        DynamicObject reimLever = getReimLever(jobId, companyId);
        if(reimLever != null){
            DynamicObject realReimSettingQuery = QueryServiceHelper.queryOne("er_reimbursesetting_rel", "id", new QFilter[]{
                    new QFilter("user", QCP.equals, userId),
                    new QFilter("company", QCP.equals, companyId)
            });
            // 存在则修改
            if(realReimSettingQuery != null){
                DynamicObject reimSettingInfo = BusinessDataServiceHelper.loadSingle(realReimSettingQuery.get("id"),"er_reimbursesetting_rel");
                reimSettingInfo.set("reimburselevel",reimLever.get("id"));
                SaveServiceHelper.save(new DynamicObject[]{ reimSettingInfo });
            }
        }
    }

    private DynamicObject getReimLever(Long jobId , Long companyId) {
        // 查询差旅级别
        List<QFilter> leverFilterList = new ArrayList<>();
        // 基础资料标准过滤条件
        leverFilterList.add(BaseDataServiceHelper.getBaseDataFilter("er_reimburselevel", companyId));
        leverFilterList.add(new QFilter("enable", QCP.equals,"1"));
        leverFilterList.add(new QFilter("nckd_joblevelhr.fbasedataid", QCP.equals,jobId));
        return QueryServiceHelper.queryOne("er_reimburselevel", "id", leverFilterList.toArray(new QFilter[0]));
    }
    /**
     * 对个人额度表的额度字段赋值
     * @param personAmountBill 个人额度表
     * @param amount 额度
     * @param startMonth 开始月份，新单则从0开始，非新单则从指定月份开始
     */
    protected void setPersonAmountData(DynamicObject personAmountBill , BigDecimal amount , int startMonth){

        for(int i = startMonth; i <= 12; i++) {

            personAmountBill.set("month" + i, amount);

        }

    }
}
